@@ -19,7 +19,7 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_WAYLAND
 
@@ -28,8 +28,9 @@
 #include <limits.h>
 #include <signal.h>
 
-#include "SDL_stdinc.h"
 #include "../../core/unix/SDL_poll.h"
+#include "../../events/SDL_events_c.h"
+#include "../SDL_clipboard_c.h"
 
 #include "SDL_waylandvideo.h"
 #include "SDL_waylanddatamanager.h"
@@ -38,7 +39,7 @@
 /* FIXME: This is arbitrary, but we want this to be less than a frame because
  * any longer can potentially spin an infinite loop of PumpEvents (!)
  */
-#define PIPE_MS_TIMEOUT 14
+#define PIPE_TIMEOUT_NS SDL_MS_TO_NS(14)
 
 static ssize_t write_pipe(int fd, const void *buffer, size_t total_length, size_t *pos)
 {
@@ -50,7 +51,7 @@ static ssize_t write_pipe(int fd, const void *buffer, size_t total_length, size_
     sigset_t old_sig_set;
     struct timespec zerotime = { 0 };
 
-    ready = SDL_IOReady(fd, SDL_IOR_WRITE, PIPE_MS_TIMEOUT);
+    ready = SDL_IOReady(fd, SDL_IOR_WRITE, PIPE_TIMEOUT_NS);
 
     sigemptyset(&sig_set);
     sigaddset(&sig_set, SIGPIPE);
@@ -86,7 +87,7 @@ static ssize_t write_pipe(int fd, const void *buffer, size_t total_length, size_
     return bytes_written;
 }
 
-static ssize_t read_pipe(int fd, void **buffer, size_t *total_length, SDL_bool null_terminate)
+static ssize_t read_pipe(int fd, void **buffer, size_t *total_length)
 {
     int ready = 0;
     void *output_buffer = NULL;
@@ -95,7 +96,7 @@ static ssize_t read_pipe(int fd, void **buffer, size_t *total_length, SDL_bool n
     ssize_t bytes_read = 0;
     size_t pos = 0;
 
-    ready = SDL_IOReady(fd, SDL_IOR_READ, PIPE_MS_TIMEOUT);
+    ready = SDL_IOReady(fd, SDL_IOR_READ, PIPE_TIMEOUT_NS);
 
     if (ready == 0) {
         bytes_read = SDL_SetError("Pipe timeout");
@@ -109,11 +110,7 @@ static ssize_t read_pipe(int fd, void **buffer, size_t *total_length, SDL_bool n
         pos = *total_length;
         *total_length += bytes_read;
 
-        if (null_terminate == SDL_TRUE) {
-            new_buffer_length = *total_length + 1;
-        } else {
-            new_buffer_length = *total_length;
-        }
+        new_buffer_length = *total_length + sizeof(Uint32);
 
         if (!*buffer) {
             output_buffer = SDL_malloc(new_buffer_length);
@@ -122,13 +119,10 @@ static ssize_t read_pipe(int fd, void **buffer, size_t *total_length, SDL_bool n
         }
 
         if (!output_buffer) {
-            bytes_read = SDL_OutOfMemory();
+            bytes_read = -1;
         } else {
             SDL_memcpy((Uint8 *)output_buffer + pos, temp, bytes_read);
-
-            if (null_terminate == SDL_TRUE) {
-                SDL_memset((Uint8 *)output_buffer + (new_buffer_length - 1), 0, 1);
-            }
+            SDL_memset((Uint8 *)output_buffer + (new_buffer_length - sizeof(Uint32)), 0, sizeof(Uint32));
 
             *buffer = output_buffer;
         }
@@ -137,31 +131,6 @@ static ssize_t read_pipe(int fd, void **buffer, size_t *total_length, SDL_bool n
     return bytes_read;
 }
 
-#define MIME_LIST_SIZE 4
-
-static const char *mime_conversion_list[MIME_LIST_SIZE][2] = {
-    { "text/plain", TEXT_MIME },
-    { "TEXT", TEXT_MIME },
-    { "UTF8_STRING", TEXT_MIME },
-    { "STRING", TEXT_MIME }
-};
-
-const char *Wayland_convert_mime_type(const char *mime_type)
-{
-    const char *found = mime_type;
-
-    size_t index = 0;
-
-    for (index = 0; index < MIME_LIST_SIZE; ++index) {
-        if (SDL_strcmp(mime_conversion_list[index][0], mime_type) == 0) {
-            found = mime_conversion_list[index][1];
-            break;
-        }
-    }
-
-    return found;
-}
-
 static SDL_MimeDataList *mime_data_list_find(struct wl_list *list,
                                              const char *mime_type)
 {
@@ -177,11 +146,11 @@ static SDL_MimeDataList *mime_data_list_find(struct wl_list *list,
     return found;
 }
 
-static int mime_data_list_add(struct wl_list *list,
+static bool mime_data_list_add(struct wl_list *list,
                               const char *mime_type,
                               const void *buffer, size_t length)
 {
-    int status = 0;
+    bool result = true;
     size_t mime_type_length = 0;
     SDL_MimeDataList *mime_data = NULL;
     void *internal_buffer = NULL;
@@ -189,7 +158,7 @@ static int mime_data_list_add(struct wl_list *list,
     if (buffer) {
         internal_buffer = SDL_malloc(length);
         if (!internal_buffer) {
-            return SDL_OutOfMemory();
+            return false;
         }
         SDL_memcpy(internal_buffer, buffer, length);
     }
@@ -199,14 +168,14 @@ static int mime_data_list_add(struct wl_list *list,
     if (!mime_data) {
         mime_data = SDL_calloc(1, sizeof(*mime_data));
         if (!mime_data) {
-            status = SDL_OutOfMemory();
+            result = false;
         } else {
             WAYLAND_wl_list_insert(list, &(mime_data->link));
 
             mime_type_length = SDL_strlen(mime_type) + 1;
             mime_data->mime_type = SDL_malloc(mime_type_length);
             if (!mime_data->mime_type) {
-                status = SDL_OutOfMemory();
+                result = false;
             } else {
                 SDL_memcpy(mime_data->mime_type, mime_type, mime_type_length);
             }
@@ -223,7 +192,7 @@ static int mime_data_list_add(struct wl_list *list,
         SDL_free(internal_buffer);
     }
 
-    return status;
+    return result;
 }
 
 static void mime_data_list_free(struct wl_list *list)
@@ -231,8 +200,7 @@ static void mime_data_list_free(struct wl_list *list)
     SDL_MimeDataList *mime_data = NULL;
     SDL_MimeDataList *next = NULL;
 
-    wl_list_for_each_safe(mime_data, next, list, link)
-    {
+    wl_list_for_each_safe (mime_data, next, list, link) {
         if (mime_data->data) {
             SDL_free(mime_data->data);
         }
@@ -243,144 +211,109 @@ static void mime_data_list_free(struct wl_list *list)
     }
 }
 
-static ssize_t Wayland_source_send(SDL_MimeDataList *mime_data, const char *mime_type, int fd)
+static size_t Wayland_send_data(const void *data, size_t length, int fd)
 {
-    size_t written_bytes = 0;
-    ssize_t status = 0;
+    size_t result = 0;
 
-    if (!mime_data || !mime_data->data) {
-        status = SDL_SetError("Invalid mime type");
-        close(fd);
-    } else {
-        while (write_pipe(fd,
-                          mime_data->data,
-                          mime_data->length,
-                          &written_bytes) > 0) {
+    if (length > 0 && data) {
+        while (write_pipe(fd, data, length, &result) > 0) {
+            // Just keep spinning
         }
-        close(fd);
-        status = written_bytes;
     }
-    return status;
+    close(fd);
+
+    return result;
 }
 
 ssize_t Wayland_data_source_send(SDL_WaylandDataSource *source, const char *mime_type, int fd)
 {
-    SDL_MimeDataList *mime_data = NULL;
+    const void *data = NULL;
+    size_t length = 0;
 
-    mime_type = Wayland_convert_mime_type(mime_type);
-    mime_data = mime_data_list_find(&source->mimes,
-                                    mime_type);
+    if (source->callback) {
+        data = source->callback(source->userdata.data, mime_type, &length);
+    }
 
-    return Wayland_source_send(mime_data, mime_type, fd);
+    return Wayland_send_data(data, length, fd);
 }
 
 ssize_t Wayland_primary_selection_source_send(SDL_WaylandPrimarySelectionSource *source, const char *mime_type, int fd)
 {
-    SDL_MimeDataList *mime_data = NULL;
+    const void *data = NULL;
+    size_t length = 0;
 
-    mime_type = Wayland_convert_mime_type(mime_type);
-    mime_data = mime_data_list_find(&source->mimes,
-                                    mime_type);
-
-    return Wayland_source_send(mime_data, mime_type, fd);
-}
-
-int Wayland_data_source_add_data(SDL_WaylandDataSource *source,
-                                 const char *mime_type,
-                                 const void *buffer,
-                                 size_t length)
-{
-    return mime_data_list_add(&source->mimes, mime_type, buffer, length);
-}
-
-int Wayland_primary_selection_source_add_data(SDL_WaylandPrimarySelectionSource *source,
-                                              const char *mime_type,
-                                              const void *buffer,
-                                              size_t length)
-{
-    return mime_data_list_add(&source->mimes, mime_type, buffer, length);
-}
-
-SDL_bool Wayland_data_source_has_mime(SDL_WaylandDataSource *source,
-                                      const char *mime_type)
-{
-    SDL_bool found = SDL_FALSE;
-
-    if (source) {
-        found = mime_data_list_find(&source->mimes, mime_type) != NULL;
+    if (source->callback) {
+        data = source->callback(source->userdata.data, mime_type, &length);
     }
-    return found;
+
+    return Wayland_send_data(data, length, fd);
 }
 
-SDL_bool Wayland_primary_selection_source_has_mime(SDL_WaylandPrimarySelectionSource *source,
-                                                   const char *mime_type)
+void Wayland_data_source_set_callback(SDL_WaylandDataSource *source,
+                                      SDL_ClipboardDataCallback callback,
+                                      void *userdata,
+                                      Uint32 sequence)
 {
-    SDL_bool found = SDL_FALSE;
-
     if (source) {
-        found = mime_data_list_find(&source->mimes, mime_type) != NULL;
+        source->callback = callback;
+        source->userdata.sequence = sequence;
+        source->userdata.data = userdata;
     }
-    return found;
 }
 
-static void *Wayland_source_get_data(SDL_MimeDataList *mime_data,
-                                     size_t *length,
-                                     SDL_bool null_terminate)
+void Wayland_primary_selection_source_set_callback(SDL_WaylandPrimarySelectionSource *source,
+                                                  SDL_ClipboardDataCallback callback,
+                                                  void *userdata)
 {
-    void *buffer = NULL;
+    if (source) {
+        source->callback = callback;
+        source->userdata.sequence = 0;
+        source->userdata.data = userdata;
+    }
+}
 
-    if (mime_data && mime_data->length > 0) {
-        size_t buffer_length = mime_data->length;
-
-        if (null_terminate == SDL_TRUE) {
-            ++buffer_length;
-        }
-        buffer = SDL_malloc(buffer_length);
-        if (!buffer) {
-            *length = SDL_OutOfMemory();
-        } else {
-            *length = mime_data->length;
-            SDL_memcpy(buffer, mime_data->data, mime_data->length);
-            if (null_terminate) {
-                *((Uint8 *)buffer + mime_data->length) = 0;
-            }
+static void *Wayland_clone_data_buffer(const void *buffer, size_t *len)
+{
+    void *clone = NULL;
+    if (*len > 0 && buffer) {
+        clone = SDL_malloc((*len)+sizeof(Uint32));
+        if (clone) {
+            SDL_memcpy(clone, buffer, *len);
+            SDL_memset((Uint8 *)clone + *len, 0, sizeof(Uint32));
         }
     }
-
-    return buffer;
+    return clone;
 }
 
 void *Wayland_data_source_get_data(SDL_WaylandDataSource *source,
-                                   size_t *length, const char *mime_type,
-                                   SDL_bool null_terminate)
+                                   const char *mime_type, size_t *length)
 {
-    SDL_MimeDataList *mime_data = NULL;
     void *buffer = NULL;
+    const void *internal_buffer;
     *length = 0;
 
     if (!source) {
         SDL_SetError("Invalid data source");
-    } else {
-        mime_data = mime_data_list_find(&source->mimes, mime_type);
-        buffer = Wayland_source_get_data(mime_data, length, null_terminate);
+    } else if (source->callback) {
+        internal_buffer = source->callback(source->userdata.data, mime_type, length);
+        buffer = Wayland_clone_data_buffer(internal_buffer, length);
     }
 
     return buffer;
 }
 
 void *Wayland_primary_selection_source_get_data(SDL_WaylandPrimarySelectionSource *source,
-                                                size_t *length, const char *mime_type,
-                                                SDL_bool null_terminate)
+                                                const char *mime_type, size_t *length)
 {
-    SDL_MimeDataList *mime_data = NULL;
     void *buffer = NULL;
+    const void *internal_buffer;
     *length = 0;
 
     if (!source) {
         SDL_SetError("Invalid primary selection source");
-    } else {
-        mime_data = mime_data_list_find(&source->mimes, mime_type);
-        buffer = Wayland_source_get_data(mime_data, length, null_terminate);
+    } else if (source->callback) {
+        internal_buffer = source->callback(source->userdata.data, mime_type, length);
+        buffer = Wayland_clone_data_buffer(internal_buffer, length);
     }
 
     return buffer;
@@ -394,7 +327,11 @@ void Wayland_data_source_destroy(SDL_WaylandDataSource *source)
             data_device->selection_source = NULL;
         }
         wl_data_source_destroy(source->source);
-        mime_data_list_free(&source->mimes);
+        if (source->userdata.sequence) {
+            SDL_CancelClipboardData(source->userdata.sequence);
+        } else {
+            SDL_free(source->userdata.data);
+        }
         SDL_free(source);
     }
 }
@@ -407,14 +344,15 @@ void Wayland_primary_selection_source_destroy(SDL_WaylandPrimarySelectionSource
             primary_selection_device->selection_source = NULL;
         }
         zwp_primary_selection_source_v1_destroy(source->source);
-        mime_data_list_free(&source->mimes);
+        if (source->userdata.sequence == 0) {
+            SDL_free(source->userdata.data);
+        }
         SDL_free(source);
     }
 }
 
 void *Wayland_data_offer_receive(SDL_WaylandDataOffer *offer,
-                                 size_t *length, const char *mime_type,
-                                 SDL_bool null_terminate)
+                                 const char *mime_type, size_t *length)
 {
     SDL_WaylandDataDevice *data_device = NULL;
 
@@ -434,21 +372,23 @@ void *Wayland_data_offer_receive(SDL_WaylandDataOffer *offer,
     } else {
         wl_data_offer_receive(offer->offer, mime_type, pipefd[1]);
 
-        /* TODO: Needs pump and flush? */
+        // TODO: Needs pump and flush?
         WAYLAND_wl_display_flush(data_device->video_data->display);
 
         close(pipefd[1]);
 
-        while (read_pipe(pipefd[0], &buffer, length, null_terminate) > 0) {
+        while (read_pipe(pipefd[0], &buffer, length) > 0) {
         }
         close(pipefd[0]);
     }
+    SDL_LogTrace(SDL_LOG_CATEGORY_INPUT,
+                 ". In Wayland_data_offer_receive for '%s', buffer (%zu) at %p",
+                 mime_type, *length, buffer);
     return buffer;
 }
 
 void *Wayland_primary_selection_offer_receive(SDL_WaylandPrimarySelectionOffer *offer,
-                                              size_t *length, const char *mime_type,
-                                              SDL_bool null_terminate)
+                                              const char *mime_type, size_t *length)
 {
     SDL_WaylandPrimarySelectionDevice *primary_selection_device = NULL;
 
@@ -468,34 +408,37 @@ void *Wayland_primary_selection_offer_receive(SDL_WaylandPrimarySelectionOffer *
     } else {
         zwp_primary_selection_offer_v1_receive(offer->offer, mime_type, pipefd[1]);
 
-        /* TODO: Needs pump and flush? */
+        // TODO: Needs pump and flush?
         WAYLAND_wl_display_flush(primary_selection_device->video_data->display);
 
         close(pipefd[1]);
 
-        while (read_pipe(pipefd[0], &buffer, length, null_terminate) > 0) {
+        while (read_pipe(pipefd[0], &buffer, length) > 0) {
         }
         close(pipefd[0]);
     }
+    SDL_LogTrace(SDL_LOG_CATEGORY_INPUT,
+                 ". In Wayland_primary_selection_offer_receive for '%s', buffer (%zu) at %p",
+                 mime_type, *length, buffer);
     return buffer;
 }
 
-int Wayland_data_offer_add_mime(SDL_WaylandDataOffer *offer,
+bool Wayland_data_offer_add_mime(SDL_WaylandDataOffer *offer,
                                 const char *mime_type)
 {
     return mime_data_list_add(&offer->mimes, mime_type, NULL, 0);
 }
 
-int Wayland_primary_selection_offer_add_mime(SDL_WaylandPrimarySelectionOffer *offer,
+bool Wayland_primary_selection_offer_add_mime(SDL_WaylandPrimarySelectionOffer *offer,
                                              const char *mime_type)
 {
     return mime_data_list_add(&offer->mimes, mime_type, NULL, 0);
 }
 
-SDL_bool Wayland_data_offer_has_mime(SDL_WaylandDataOffer *offer,
+bool Wayland_data_offer_has_mime(SDL_WaylandDataOffer *offer,
                                      const char *mime_type)
 {
-    SDL_bool found = SDL_FALSE;
+    bool found = false;
 
     if (offer) {
         found = mime_data_list_find(&offer->mimes, mime_type) != NULL;
@@ -503,10 +446,10 @@ SDL_bool Wayland_data_offer_has_mime(SDL_WaylandDataOffer *offer,
     return found;
 }
 
-SDL_bool Wayland_primary_selection_offer_has_mime(SDL_WaylandPrimarySelectionOffer *offer,
+bool Wayland_primary_selection_offer_has_mime(SDL_WaylandPrimarySelectionOffer *offer,
                                                   const char *mime_type)
 {
-    SDL_bool found = SDL_FALSE;
+    bool found = false;
 
     if (offer) {
         found = mime_data_list_find(&offer->mimes, mime_type) != NULL;
@@ -532,70 +475,61 @@ void Wayland_primary_selection_offer_destroy(SDL_WaylandPrimarySelectionOffer *o
     }
 }
 
-int Wayland_data_device_clear_selection(SDL_WaylandDataDevice *data_device)
+bool Wayland_data_device_clear_selection(SDL_WaylandDataDevice *data_device)
 {
-    int status = 0;
+    bool result = true;
 
     if (!data_device || !data_device->data_device) {
-        status = SDL_SetError("Invalid Data Device");
+        result = SDL_SetError("Invalid Data Device");
     } else if (data_device->selection_source) {
         wl_data_device_set_selection(data_device->data_device, NULL, 0);
         Wayland_data_source_destroy(data_device->selection_source);
         data_device->selection_source = NULL;
     }
-    return status;
+    return result;
 }
 
-int Wayland_primary_selection_device_clear_selection(SDL_WaylandPrimarySelectionDevice *primary_selection_device)
+bool Wayland_primary_selection_device_clear_selection(SDL_WaylandPrimarySelectionDevice *primary_selection_device)
 {
-    int status = 0;
+    bool result = true;
 
     if (!primary_selection_device || !primary_selection_device->primary_selection_device) {
-        status = SDL_SetError("Invalid Primary Selection Device");
+        result = SDL_SetError("Invalid Primary Selection Device");
     } else if (primary_selection_device->selection_source) {
         zwp_primary_selection_device_v1_set_selection(primary_selection_device->primary_selection_device,
                                                       NULL, 0);
         Wayland_primary_selection_source_destroy(primary_selection_device->selection_source);
         primary_selection_device->selection_source = NULL;
     }
-    return status;
+    return result;
 }
 
-int Wayland_data_device_set_selection(SDL_WaylandDataDevice *data_device,
-                                      SDL_WaylandDataSource *source)
+bool Wayland_data_device_set_selection(SDL_WaylandDataDevice *data_device,
+                                       SDL_WaylandDataSource *source,
+                                       const char **mime_types,
+                                       size_t mime_count)
 {
-    int status = 0;
-    size_t num_offers = 0;
-    size_t index = 0;
+    bool result = true;
 
     if (!data_device) {
-        status = SDL_SetError("Invalid Data Device");
+        result = SDL_SetError("Invalid Data Device");
     } else if (!source) {
-        status = SDL_SetError("Invalid source");
+        result = SDL_SetError("Invalid source");
     } else {
-        SDL_MimeDataList *mime_data = NULL;
+        size_t index = 0;
+        const char *mime_type;
 
-        wl_list_for_each (mime_data, &(source->mimes), link) {
+        for (index = 0; index < mime_count; ++index) {
+            mime_type = mime_types[index];
             wl_data_source_offer(source->source,
-                                 mime_data->mime_type);
-
-            /* TODO - Improve system for multiple mime types to same data */
-            for (index = 0; index < MIME_LIST_SIZE; ++index) {
-                if (SDL_strcmp(mime_conversion_list[index][1], mime_data->mime_type) == 0) {
-                    wl_data_source_offer(source->source,
-                                         mime_conversion_list[index][0]);
-                }
-            }
-            /* */
-
-            ++num_offers;
+                                 mime_type);
         }
 
-        if (num_offers == 0) {
+        if (index == 0) {
             Wayland_data_device_clear_selection(data_device);
-            status = SDL_SetError("No mime data");
+            result = SDL_SetError("No mime data");
         } else {
-            /* Only set if there is a valid serial if not set it later */
+            // Only set if there is a valid serial if not set it later
             if (data_device->selection_serial != 0) {
                 wl_data_device_set_selection(data_device->data_device,
                                              source->source,
@@ -609,44 +543,34 @@ int Wayland_data_device_set_selection(SDL_WaylandDataDevice *data_device,
         }
     }
 
-    return status;
+    return result;
 }
 
-int Wayland_primary_selection_device_set_selection(SDL_WaylandPrimarySelectionDevice *primary_selection_device,
-                                                   SDL_WaylandPrimarySelectionSource *source)
+bool Wayland_primary_selection_device_set_selection(SDL_WaylandPrimarySelectionDevice *primary_selection_device,
+                                                   SDL_WaylandPrimarySelectionSource *source,
+                                                   const char **mime_types,
+                                                   size_t mime_count)
 {
-    int status = 0;
-    size_t num_offers = 0;
-    size_t index = 0;
+    bool result = true;
 
     if (!primary_selection_device) {
-        status = SDL_SetError("Invalid Primary Selection Device");
+        result = SDL_SetError("Invalid Primary Selection Device");
     } else if (!source) {
-        status = SDL_SetError("Invalid source");
+        result = SDL_SetError("Invalid source");
     } else {
-        SDL_MimeDataList *mime_data = NULL;
+        size_t index = 0;
+        const char *mime_type = mime_types[index];
 
-        wl_list_for_each (mime_data, &(source->mimes), link) {
-            zwp_primary_selection_source_v1_offer(source->source,
-                                                  mime_data->mime_type);
-
-            /* TODO - Improve system for multiple mime types to same data */
-            for (index = 0; index < MIME_LIST_SIZE; ++index) {
-                if (SDL_strcmp(mime_conversion_list[index][1], mime_data->mime_type) == 0) {
-                    zwp_primary_selection_source_v1_offer(source->source,
-                                                          mime_conversion_list[index][0]);
-                }
-            }
-            /* */
-
-            ++num_offers;
+        for (index = 0; index < mime_count; ++index) {
+            mime_type = mime_types[index];
+            zwp_primary_selection_source_v1_offer(source->source, mime_type);
         }
 
-        if (num_offers == 0) {
+        if (index == 0) {
             Wayland_primary_selection_device_clear_selection(primary_selection_device);
-            status = SDL_SetError("No mime data");
+            result = SDL_SetError("No mime data");
         } else {
-            /* Only set if there is a valid serial if not set it later */
+            // Only set if there is a valid serial if not set it later
             if (primary_selection_device->selection_serial != 0) {
                 zwp_primary_selection_device_v1_set_selection(primary_selection_device->primary_selection_device,
                                                               source->source,
@@ -660,17 +584,13 @@ int Wayland_primary_selection_device_set_selection(SDL_WaylandPrimarySelectionDe
         }
     }
 
-    return status;
+    return result;
 }
 
-int Wayland_data_device_set_serial(SDL_WaylandDataDevice *data_device,
-                                   uint32_t serial)
+void Wayland_data_device_set_serial(SDL_WaylandDataDevice *data_device, uint32_t serial)
 {
-    int status = -1;
     if (data_device) {
-        status = 0;
-
-        /* If there was no serial and there is a pending selection set it now. */
+        // If there was no serial and there is a pending selection set it now.
         if (data_device->selection_serial == 0 && data_device->selection_source) {
             wl_data_device_set_selection(data_device->data_device,
                                          data_device->selection_source->source,
@@ -679,18 +599,13 @@ int Wayland_data_device_set_serial(SDL_WaylandDataDevice *data_device,
 
         data_device->selection_serial = serial;
     }
-
-    return status;
 }
 
-int Wayland_primary_selection_device_set_serial(SDL_WaylandPrimarySelectionDevice *primary_selection_device,
-                                                uint32_t serial)
+void Wayland_primary_selection_device_set_serial(SDL_WaylandPrimarySelectionDevice *primary_selection_device,
+                                                 uint32_t serial)
 {
-    int status = -1;
     if (primary_selection_device) {
-        status = 0;
-
-        /* If there was no serial and there is a pending selection set it now. */
+        // If there was no serial and there is a pending selection set it now.
         if (primary_selection_device->selection_serial == 0 && primary_selection_device->selection_source) {
             zwp_primary_selection_device_v1_set_selection(primary_selection_device->primary_selection_device,
                                                           primary_selection_device->selection_source->source,
@@ -699,10 +614,6 @@ int Wayland_primary_selection_device_set_serial(SDL_WaylandPrimarySelectionDevic
 
         primary_selection_device->selection_serial = serial;
     }
-
-    return status;
 }
 
-#endif /* SDL_VIDEO_DRIVER_WAYLAND */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_WAYLAND
